#!/bin/bash
#
# Copyright (c) 2018 SUSE LINUX GmbH, Nuernberg, Germany.

umask 022
PATH=/sbin:/bin:/usr/sbin:/usr/bin
export PATH

if [ -f /etc/default/btrfsmaintenance ] ; then
    . /etc/default/btrfsmaintenance
fi

if [ -f /etc/default/btrfsmaintenance ] ; then
    . /etc/default/btrfsmaintenance
fi

LOGIDENTIFIER='btrfs-trim'
. $(dirname $(realpath "$0"))/btrfsmaintenance-functions

{
BTRFS_TRIM_MOUNTPOINTS=$(expand_auto_mountpoint "$BTRFS_TRIM_MOUNTPOINTS")
OIFS="$IFS"
IFS=:
exec 2>&1 # redirect stderr to stdout to catch all output to log destination
for MNT in $BTRFS_TRIM_MOUNTPOINTS; do
	IFS="$OIFS"
	if ! is_btrfs "$MNT"; then
		echo "Path $MNT is not btrfs, skipping"
		continue
	fi
	#echo "Running fstrim on $MNT"
	#run_task fstrim --verbose "$MNT"

	unset DEV alignment_type

	DEV="$( awk -v MNT="$MNT" '{if ($2 == MNT) print $1}' /proc/mounts | tail -1 )"

	# lsblk also supports /dev/dm-*
	alignment_type="$( LC_ALL=C lsblk -rino ROTA "$DEV" | grep "^[[:digit:]]" | tail -1 )"

	if [ -z "$alignment_type" ] ; then
		alignment_type="$( cat /sys/block/$(basename ${DEV})/queue/rotational 2>/dev/null )"
	fi

	if [ "$alignment_type" = 0 ] ; then
		echo "Running fstrim on $MNT"
		run_task fstrim --verbose "$MNT"
    else
       echo "Skipping fstrim on $MNT because is not from a SSD device"
	fi
done

} | \
case "$BTRFS_LOG_OUTPUT" in
	stdout) cat;;
	journal) systemd-cat -t "$LOGIDENTIFIER";;
	syslog) logger -t "$LOGIDENTIFIER";;
	none) cat >/dev/null;;
	*) cat;;
esac

exit 0
